@@ -101,31 +101,48 @@ class R3sponse(object):
 		# print the response as json.
 		json=False,
 		# optionals:
-		# the log level for printing to console.
+		# the active log level.
 		log_level=0,
 		# the required log level for when printed to console (leave None to use self.log_level).
 		required_log_level=None, 
 		# save to log file.
 		save=False,
-		# save errors only (for option 2 only).
-		save_errors=False,
+		# save errors always (for options 2 & 3 only).
+		save_errors=None,
+		# the log mode (leave None for default).
+		mode=None,
 	):
+		if mode != None: mode = str(mode).lower().replace("-","").replace("_","")
+		if mode not in [None, "warning", "alert"]:
+			raise exceptions.InvalidUsage(f"{self.__traceback__(function='log', parameter='mode')}: Selected an invalid mode [{mode}], options: [None, warning, alert].")
+		def fill_message(msg, error=False):
+			if mode == None:
+				if error:
+					msg = f"Error: {msg}"
+				else:
+					msg = msg
+			elif mode == "warning":
+				msg = f"&RED&:Warning&END&: {msg}"
+			elif mode == "alert":
+				msg = f"&ORANGE&:Alert&END&: {msg}"
+			return msg
+			#
 		msg, _error_ = None, False
 		if [message,error,response] == [None,None,{}]:
-			raise ValueError("Define either parameter [message:str], [error:str] or [response:dict].")
+			raise exceptions.InvalidUsage(f"{self.__traceback__(function='log')}: Define either parameter [message:str], [error:str] or [response:dict].")
 		if response != {}:
 			if response["error"] != None: 
 				_error_ = True
 				msg = f"Error: {response['error']}"
 			else: 
 				if response.__class__.__name__ in ["Output"]: 
-					msg = str(response)
+					msg = response.output
 				else:
 					msg = response["message"]
-		elif error != None: 
-			msg = f"Error: {error}"
+		elif isinstance(error, (str, String)): 
+			msg = fill_message(error, error=True)
 		else: 
-			msg = message
+			msg = fill_message(message, error=False)
 		if required_log_level == None: required_log_level = self.log_level
 		try:
 			required_log_level = int(required_log_level)
@@ -178,57 +195,89 @@ class R3sponse(object):
 		#
 	
 	# init a response blank object.
-	def serialize(self, response={}):
-		if isinstance(response, str):
+	def serialize(self, 
+		# the response (#1) (dict) (str repr of dict) (ResponseObject) (generator) .
+		response={}, 
+		# init to response object.
+		init=True,
+	):
+
+		# check async generator.
+		if response.__class__.__name__ in ["generator"]:
+			raise ValueError("Not supported yet.")
+			return self.serialize(response=response.value, init=init)
+			#print("Converting generator.")
+			#print(f"Generator value ({response.value.__class__.__name__}) [{response.value}].")
+			#try:
+			#	value = next(response)
+			#except StopIteration as e:
+			#	return self.serialize(response=str(e), init=True)
+			#	#print(f"Generator output: [{e}].")
+			#	#response = self.serialize(response=str(e), init=True)
+			#	#print(f"Serialized generator output instance: [{response.__class__.__name__}].")
+			#	#return response
+			#except Eception as e:
+			#	return r3sponse.error(f"An error occured during the execution of generator [{response}], error: {e}")
+
+		# check ResponseObject.
+		elif response.__class__.__name__ in ["ResponseObject"]:
+			return response
+		
+		# check syst3m.console.Output.
+		elif response.__class__.__name__ in ["Output"]:
 			try:
-				response = ast.literal_eval(response)
-			except:
-				response = json.loads(response)
-		for key in list(response.keys()):
-			value = response[key]
-			no_dict = False
-			if isinstance(value, dict):
-				value = self.serialize(value)
-			else:
+				return response.response()
+			except AttributeError:
+				return response
+
+		# dict / str.
+		elif response.__class__.__name__ in ["str", "String", "dict", "Dictionary"]:
+			if response.__class__.__name__ in ["str", "String"]:
 				try:
-					value = ast.literal_eval(value)
-				except:
 					try:
-						value = json.loads(value)
+						response = ast.literal_eval(response)
 					except:
-						no_dict = True
-				if no_dict == False and isinstance(value, dict):
+						response = json.loads(response)
+				except Exception as e: 
+					raise exceptions.JsonDecodeError(f"Unable to parse a dictionary from [{response}], {e}.")
+			for key in list(response.keys()):
+				value = response[key]
+				no_dict = False
+				if isinstance(value, dict):
 					value = self.serialize(value)
-			if no_dict == False: a=1
-			elif value in [True, "True", "true", "TRUE"]: value = True
-			elif value in [False, "False", "false", "FALSE"]: value = False
-			elif isinstance(value, str):
-				if "." in value:
-					try: value = float(value)
-					except: a=1
-				else:
-					try: value = int(value)
-					except: a=1
-			response[key] = value
-		return response
+				elif value in [None, "None", "none", "null"]: value = None
+				elif value in [True, "True", "true", "TRUE"]: value = True
+				elif value in [False, "False", "false", "FALSE"]: value = False
+				elif isinstance(value, str):
+					if "." in value:
+						try: value = float(value)
+						except: a=1
+					else:
+						try: value = int(value)
+						except: a=1
+				response[key] = value
+
+		# invalid.
+		else:
+			raise exceptions.InvalidUsage(f"The parameter [response] must be [str, String, dict, Dictionary, generator] not [{response.__class__.__name__}].")
+
+		# return dict.
+		if init:
+			return ResponseObject(response)
+		else:
+			return response
+
+		#
 	def response(self, 
-		# the blank response (dict) (#1).
+		# the blank response (dict, str, generator) (#1).
 		response={
 			"success":False,
 			"message":None,
 			"error":None,
 		},
-		# the safe boolean (serialize from dumped json).
-		safe=False,
 	):
-		# do not serialize (always keep default).
-		if not safe:
-			return ResponseObject(response)
-
-		# serialize the response (serialize from dumped json).
-		else:
-			return self.response(self.serialize(response))
-
+		# serialize shortcut.
+		return self.serialize(response, init=True)
 		#
 
 	# system functions.
@@ -261,15 +310,15 @@ class R3sponse(object):
 class Parameters(object):
 	def __init__(self):
 		a=1
-
+		#
 	# get request parameters.
 	def get(self, 
 		# the django request (1).
 		request=None, 
 		# the identifiers (#2).
 		#	str instance: return the parameters value.
-		#	list instance: return a parameters object class & return an error response when a parameter is undefined.
-		#	dict instance: return a parameters object class & return the parameter's value from the dict as a default when undefined.
+		#	list instance: return a parameters object & return an error response when a parameter is undefined.
+		#	dict instance: return a parameters object & return the parameter's value from the dict as a default when undefined.
 		parameters=[], 
 		# default return value (dict instance of parameters overwrites the default parameter).
 		default=None, 
@@ -478,11 +527,36 @@ class ResponseObject(object):
 		# except if not present.
 		except AttributeError: a=1
 	# clean default values.
-	def clean(self):
-		for i in ["error", "message", "success"]:
-			del self[i]
-			#except: a=1
-		return self
+	def clean(self, 
+		# the clean options, select * for all, options: [traceback].
+		options=["*"],
+		# serialize to ResponseObject (with serialize False the ResponseObject's values are not updated).
+		serialize=True,
+	):
+		
+		# options.
+		if options in ["*", "all"]: options = ["*"]
+		_options = []
+		for i in options: _options.append(i.lower())
+		options = _options
+		
+		# clean.
+		attributes = self.dict(json=False)
+		remove = ["error", "message", "success"]
+		if "*" in options or "traceback" in options: remove += ["_traceback_", "__traceback__", "raw_traceback", "_raw_traceback_", "__raw_traceback__"]
+		for i in remove:
+			try:
+				if serialize:
+					del self[str(i)]
+				else:
+					del attributes[str(i)]
+			except KeyError: a=1
+		if serialize:
+			return self
+		else:
+			return attributes
+
+		#
 	# assign dict.
 	def assign(self, dictionary):
 		if isinstance(dictionary, (dict, Dictionary)):
@@ -722,6 +796,10 @@ class ResponseObject(object):
 	# return raw data.
 	def raw(self):
 		return self.dict()
+	# return response self for syst3m.console.Output and other objects that init ResponseObject as self and want it to be converted to response.
+	def response(self):
+		return self
+		#
 
 # initialized objects.
 # must be initialized as r3sponse since Parameters requires it.
